@@ -19,19 +19,21 @@
 #include "kstandardaction.h"
 #include "kstandardaction_p.h"
 #include "moc_kstandardaction_p.cpp"
+#include "kconfigwidgets_debug.h"
 
-#include <QtCore/QMutableStringListIterator>
+#include <QMutableStringListIterator>
 #include <QToolButton>
 
 #include <QApplication>
+#include <QLayout>
+#include <QMainWindow>
+#include <QMenuBar>
 #include <kaboutdata.h>
 #include <klocalizedstring.h>
 #include <kstandardshortcut.h>
 #include <kacceleratormanager.h>
 
 #include "kdualaction.h"
-#include "krecentfilesaction.h"
-#include "ktogglefullscreenaction.h"
 #include "kpastetextaction.h"
 
 namespace KStandardAction
@@ -63,13 +65,75 @@ QList<StandardAction> actionIds()
     return result;
 }
 
-KCONFIGWIDGETS_EXPORT KStandardShortcut::StandardShortcut shortcutForActionId(StandardAction id)
+KStandardShortcut::StandardShortcut shortcutForActionId(StandardAction id)
 {
     const KStandardActionInfo *pInfo = infoPtr(id);
     return (pInfo) ? pInfo->idAccel : KStandardShortcut::AccelNone;
 }
 
-QAction *create(StandardAction id, const QObject *recvr, const char *slot, QObject *parent)
+class ShowMenubarActionFilter : public QObject
+{
+public:
+    ShowMenubarActionFilter(QAction *parent)
+        : QObject(parent)
+        , wasNative(false)
+        , wasChecked(false)
+        , wasVisible(false)
+    {
+    }
+
+    bool eventFilter(QObject * /*watched*/, QEvent *e) override
+    {
+        if (e->type() == QEvent::Show) {
+            updateAction();
+        }
+        return false;
+    }
+
+    void updateAction()
+    {
+        bool allMenuBarsNative = true;
+        bool hasAnyMenuBar = false;
+        foreach(QWidget *w, qApp->topLevelWidgets()) {
+            QMainWindow *mw = qobject_cast<QMainWindow*>(w);
+            if (mw) {
+                mw->installEventFilter(this); // this is just in case a new main window appeared
+                                                // if we were filtering it already it is almost a noop
+                if (mw->layout() && mw->layout()->menuBar()) {
+                    QMenuBar *mb = qobject_cast<QMenuBar*>(mw->layout()->menuBar());
+                    if (mb) {
+                        hasAnyMenuBar = true;
+                        if (!mb->isNativeMenuBar()) {
+                            allMenuBarsNative = false;
+                        }
+                    }
+                }
+            }
+        }
+
+        if (!hasAnyMenuBar)
+            return;
+
+        QAction *showMenubarAction = static_cast<QAction *>(parent());
+        if (allMenuBarsNative && !wasNative) {
+            wasNative = true;
+            wasChecked = showMenubarAction->isChecked();
+            wasVisible = showMenubarAction->isVisible();
+
+            showMenubarAction->setChecked(true);
+            showMenubarAction->setVisible(false);
+        } else if (!allMenuBarsNative && wasNative) {
+            showMenubarAction->setChecked(wasChecked);
+            showMenubarAction->setVisible(wasVisible);
+        }
+    }
+
+    bool wasNative;
+    bool wasChecked;
+    bool wasVisible;
+};
+
+QAction* _k_createInternal(StandardAction id, QObject *parent)
 {
     static bool stdNamesInitialized = false;
 
@@ -78,10 +142,10 @@ QAction *create(StandardAction id, const QObject *recvr, const char *slot, QObje
         stdNamesInitialized = true;
     }
 
-    QAction *pAction = 0;
+    QAction *pAction = nullptr;
     const KStandardActionInfo *pInfo = infoPtr(id);
 
-    // qDebug() << "KStandardAction::create( " << id << "=" << (pInfo ? pInfo->psName : (const char*)0) << ", " << parent << " )"; // ellis
+    // qCDebug(KCONFIG_WIDGETS_LOG) << "KStandardAction::create( " << id << "=" << (pInfo ? pInfo->psName : (const char*)0) << ", " << parent << " )"; // ellis
 
     if (pInfo) {
         QString sLabel, iconName = pInfo->psIconName;
@@ -139,19 +203,40 @@ QAction *create(StandardAction id, const QObject *recvr, const char *slot, QObje
             pAction = new KRecentFilesAction(parent);
             break;
         case ShowMenubar:
+        {
+            pAction = new KToggleAction(parent);
+            pAction->setWhatsThis(i18n("Show Menubar<p>"
+                                  "Shows the menubar again after it has been hidden</p>"));
+            pAction->setChecked(true);
+
+            ShowMenubarActionFilter *mf = new ShowMenubarActionFilter(pAction);
+            foreach(QWidget *w, qApp->topLevelWidgets()) {
+                if (qobject_cast<QMainWindow*>(w)) {
+                    w->installEventFilter(mf);
+                }
+            }
+            mf->updateAction();
+            break;
+        }
         case ShowToolbar:
+            pAction = new KToggleAction(parent);
+            pAction->setChecked(true);
+            break;
         case ShowStatusbar:
-            pAction = new QAction(parent);
-            pAction->setCheckable(true);
+            pAction = new KToggleAction(parent);
+            pAction->setWhatsThis(i18n("Show Statusbar<p>"
+                                  "Shows the statusbar, which is the bar at the bottom of the window used for status information.</p>"));
             pAction->setChecked(true);
             break;
         case FullScreen:
             pAction = new KToggleFullScreenAction(parent);
             pAction->setCheckable(true);
             break;
+#ifndef KDE_NO_DEPRECATED
         case PasteText:
             pAction = new KPasteTextAction(parent);
             break;
+#endif
         // Same as default, but with the app icon
         case AboutApp:
         {
@@ -171,6 +256,10 @@ QAction *create(StandardAction id, const QObject *recvr, const char *slot, QObje
             break;
         }
 
+        // Set the text before setting the MenuRole, as on OS X setText will do some heuristic role guessing.
+        // This ensures user menu items get the intended role out of the list below.
+        pAction->setText(sLabel);
+
         switch (id) {
         case Quit:
             pAction->setMenuRole(QAction::QuitRole);
@@ -189,7 +278,6 @@ QAction *create(StandardAction id, const QObject *recvr, const char *slot, QObje
             break;
         }
 
-        pAction->setText(sLabel);
         if (pInfo->psToolTip) {
             pAction->setToolTip(i18n(pInfo->psToolTip));
         }
@@ -205,6 +293,16 @@ QAction *create(StandardAction id, const QObject *recvr, const char *slot, QObje
         pAction->setObjectName(pInfo->psName);
     }
 
+    if (pAction && parent && parent->inherits("KActionCollection")) {
+        QMetaObject::invokeMethod(parent, "addAction", Q_ARG(QString, pAction->objectName()), Q_ARG(QAction *, pAction));
+    }
+
+    return pAction;
+}
+
+QAction *create(StandardAction id, const QObject *recvr, const char *slot, QObject *parent)
+{
+    QAction* pAction = _k_createInternal(id, parent);
     if (recvr && slot) {
         if (id == OpenRecent) {
             // FIXME QAction port: probably a good idea to find a cleaner way to do this
@@ -216,18 +314,13 @@ QAction *create(StandardAction id, const QObject *recvr, const char *slot, QObje
             QObject::connect(pAction, SIGNAL(triggered(bool)), recvr, slot);
         }
     }
-
-    if (pAction && parent && parent->inherits("KActionCollection")) {
-        QMetaObject::invokeMethod(parent, "addAction", Q_ARG(QString, pAction->objectName()), Q_ARG(QAction *, pAction));
-    }
-
     return pAction;
 }
 
 const char *name(StandardAction id)
 {
     const KStandardActionInfo *pInfo = infoPtr(id);
-    return (pInfo) ? pInfo->psName : 0;
+    return (pInfo) ? pInfo->psName : nullptr;
 }
 
 QAction *openNew(const QObject *recvr, const char *slot, QObject *parent)
@@ -310,10 +403,12 @@ QAction *paste(const QObject *recvr, const char *slot, QObject *parent)
     return KStandardAction::create(Paste, recvr, slot, parent);
 }
 
+#ifndef KDE_NO_DEPRECATED
 QAction *pasteText(const QObject *recvr, const char *slot, QObject *parent)
 {
     return KStandardAction::create(PasteText, recvr, slot, parent);
 }
+#endif
 
 QAction *clear(const QObject *recvr, const char *slot, QObject *parent)
 {
@@ -474,7 +569,7 @@ static QAction *buildAutomaticAction(QObject *parent, StandardAction id, const c
 {
     const KStandardActionInfo *p = infoPtr(id);
     if (!p) {
-        return 0;
+        return nullptr;
     }
 
     AutomaticAction *action = new AutomaticAction(
@@ -523,49 +618,16 @@ QAction *selectAll(QObject *parent)
 
 KToggleAction *showMenubar(const QObject *recvr, const char *slot, QObject *parent)
 {
-    KToggleAction *ret = new KToggleAction(i18n("Show &Menubar"), parent);
-    ret->setObjectName(name(ShowMenubar));
-    ret->setIcon(QIcon::fromTheme(QStringLiteral("show-menu")));
-
-    // emulate KActionCollection::setDefaultShortcuts to allow the use of "configure shortcuts"
-    ret->setShortcuts(KStandardShortcut::shortcut(KStandardShortcut::ShowMenubar));
-    ret->setProperty("defaultShortcuts", QVariant::fromValue(KStandardShortcut::shortcut(KStandardShortcut::ShowMenubar)));
-
-    ret->setWhatsThis(i18n("Show Menubar<p>"
-                           "Shows the menubar again after it has been hidden</p>"));
-
-    ret->setChecked(true);
-
-    if (recvr && slot) {
-        QObject::connect(ret, SIGNAL(triggered(bool)), recvr, slot);
-    }
-
-    if (parent && parent->inherits("KActionCollection")) {
-        QMetaObject::invokeMethod(parent, "addAction", Q_ARG(QString, ret->objectName()), Q_ARG(QAction *, ret));
-    }
-
-    return ret;
+    QAction* ret = KStandardAction::create(ShowMenubar, recvr, slot, parent);
+    Q_ASSERT(qobject_cast<KToggleAction *>(ret));
+    return static_cast<KToggleAction *>(ret);
 }
 
 KToggleAction *showStatusbar(const QObject *recvr, const char *slot, QObject *parent)
 {
-    KToggleAction *ret = new KToggleAction(i18n("Show St&atusbar"), parent);
-    ret->setObjectName(name(ShowStatusbar));
-
-    ret->setWhatsThis(i18n("Show Statusbar<p>"
-                           "Shows the statusbar, which is the bar at the bottom of the window used for status information.</p>"));
-
-    ret->setChecked(true);
-
-    if (recvr && slot) {
-        QObject::connect(ret, SIGNAL(triggered(bool)), recvr, slot);
-    }
-
-    if (parent && parent->inherits("KActionCollection")) {
-        QMetaObject::invokeMethod(parent, "addAction", Q_ARG(QString, ret->objectName()), Q_ARG(QAction *, ret));
-    }
-
-    return ret;
+    QAction* ret = KStandardAction::create(ShowStatusbar, recvr, slot, parent);
+    Q_ASSERT(qobject_cast<KToggleAction *>(ret));
+    return static_cast<KToggleAction *>(ret);
 }
 
 KToggleFullScreenAction *fullScreen(const QObject *recvr, const char *slot, QWidget *window, QObject *parent)
@@ -577,10 +639,12 @@ KToggleFullScreenAction *fullScreen(const QObject *recvr, const char *slot, QWid
     return ret;
 }
 
+#ifndef KDE_NO_DEPRECATED
 QAction *saveOptions(const QObject *recvr, const char *slot, QObject *parent)
 {
     return KStandardAction::create(SaveOptions, recvr, slot, parent);
 }
+#endif
 
 QAction *keyBindings(const QObject *recvr, const char *slot, QObject *parent)
 {
@@ -602,10 +666,12 @@ QAction *configureNotifications(const QObject *recvr, const char *slot, QObject
     return KStandardAction::create(ConfigureNotifications, recvr, slot, parent);
 }
 
+#ifndef KDE_NO_DEPRECATED
 QAction *help(const QObject *recvr, const char *slot, QObject *parent)
 {
     return KStandardAction::create(Help, recvr, slot, parent);
 }
+#endif
 
 QAction *helpContents(const QObject *recvr, const char *slot, QObject *parent)
 {
@@ -642,5 +708,25 @@ QAction *aboutKDE(const QObject *recvr, const char *slot, QObject *parent)
     return KStandardAction::create(AboutKDE, recvr, slot, parent);
 }
 
+QAction *deleteFile(const QObject *recvr, const char *slot, QObject *parent)
+{
+    return KStandardAction::create(DeleteFile, recvr, slot, parent);
+}
+
+QAction *renameFile(const QObject *recvr, const char *slot, QObject *parent)
+{
+    return KStandardAction::create(RenameFile, recvr, slot, parent);
+}
+
+QAction *moveToTrash(const QObject *recvr, const char *slot, QObject *parent)
+{
+    return KStandardAction::create(MoveToTrash, recvr, slot, parent);
+}
+
+QAction *donate(const QObject *recvr, const char *slot, QObject *parent)
+{
+    return KStandardAction::create(Donate, recvr, slot, parent);
+}
+
 }
 
